
library("deSolve") # loads required package, you may have to install it first
library("bbmle") # loads required package, you may have to install it first
setwd("~/MainSources/") ## set your working directory
source("source/infection.models.r") # loads required source file from your machine
source("source/infection.nll.r") # loads required source file from your machine
real.data = read.csv("data/real.data.2.csv") # reads the rquired data from your machine

str(real.data) # to investigate the data

## divide the data sets into the control and the treatment data:
real.data.control = subset(real.data, treatment == "control")
real.data.treat = subset(real.data, treatment == "treatment")

##############################################################
## Fitting the treatment data with only the 1-pathogen model:

# create empty vectors to store the fitting results in:
r.treat.simple = vector() # for the value
r.treat.simple.se = vector() # for the standard error
r.treat.simple.z = vector() # for the z value
r.treat.simple.p = vector() # for the p-value
t0.treat.simple = vector() # for the value
t0.treat.simple.se = vector() # for the standard error
t0.treat.simple.z = vector() # for the z value
t0.treat.simple.p = vector() # for the p-value
AIC.treat.simple = vector() # for the AIC

set.seed(123) # set the seed (to make the randomization repeatable)

# the for loop to fit 100 times the control data and to select the best fit:
for(i in 1:100){
	
	# Samples a start for r ranging from 0.0001 to 15:
	r.start = runif(1,min=0.01,max=15)
	
	# Samples a start for t0 from a value near to zero to a value slightly smaller than the day of the first infection:
	t0.start = runif(1,min=0.001,max=min(real.data.treat$time.days[real.data.treat$number.infected>0])*0.8)
	
	# sets the ith value to NA or infinity (i=1 in the first loop, 2 in the second and so on):
	r.treat.simple[i] = NA
	r.treat.simple.se[i] = NA
	r.treat.simple.z[i] = NA
	r.treat.simple.p[i] = NA
	t0.treat.simple[i] = NA
	t0.treat.simple.se[i] = NA
	t0.treat.simple.z[i] = NA
	t0.treat.simple.p[i] = NA
	AIC.treat.simple[i] = Inf
	# sets the object storing the fitting results to "NULL" to avoid a wrong entry from the loop before:
	fit.treat.simple = NULL
	
	# The try function allows a continuation of the for loop, even if the fit fails. This might happen if the starting values are badly chosen.
	try({
		# The fit (see manual for details):
		fit.treat.simple = mle2(mon.inf.lag.1p.nll,
			start = list(r = r.start, t0 = t0.start),
			data = list(nI = real.data.treat$number.infected,
				tps = real.data.treat$time.days,
				Imax = real.data.treat$total.number
			),
			fixed = list(steps = 0.025), # decrease the temporal resolution of the lsoda function (increase the precisness of the curve)
			control = list(maxit=10000, reltol=1e-32) # increase the maximum number of iteration steps of the optimizer and decrease the tolerance
		)
		# save the summary statistics and the AIC to the vectors that you can display them later:
		r.treat.simple[i] = summary(fit.treat.simple)@coef[1]
		r.treat.simple.se[i] = summary(fit.treat.simple)@coef[3]
		r.treat.simple.z[i] = summary(fit.treat.simple)@coef[5]
		r.treat.simple.p[i] = summary(fit.treat.simple)@coef[7]
		t0.treat.simple[i] = summary(fit.treat.simple)@coef[2]
		t0.treat.simple.se[i] = summary(fit.treat.simple)@coef[4]
		t0.treat.simple.z[i] = summary(fit.treat.simple)@coef[6]
		t0.treat.simple.p[i] = summary(fit.treat.simple)@coef[8]
		AIC.treat.simple[i] = AIC(fit.treat.simple)
	})
	print(i) # which loop are we in?
	
}

# calculates the delta-AIC, note that comparing the likelihood would be also sufficient!
dAIC.treat.simple = AIC.treat.simple-min(AIC.treat.simple)

# displaying the fitting results of the best model:
r.treat.simple[dAIC.treat.simple==0]
r.treat.simple.se[dAIC.treat.simple==0]
r.treat.simple.z[dAIC.treat.simple==0]
r.treat.simple.p[dAIC.treat.simple==0]

t0.treat.simple[dAIC.treat.simple==0]
t0.treat.simple.se[dAIC.treat.simple==0]
t0.treat.simple.z[dAIC.treat.simple==0]
t0.treat.simple.p[dAIC.treat.simple==0]

##############################################################
## Fitting the control data with only the 1-pathogen model:

# create empty vectors to store the fitting results in:
r.control = vector() # for the value
r.control.se = vector() # for the standard error
r.control.z = vector() # for the z value
r.control.p = vector() # for the p-value
t0.control = vector() # for the value
t0.control.se = vector() # for the standard error
t0.control.z = vector() # for the z value
t0.control.p = vector() # for the p-value
AIC.control = vector() # for the AIC

set.seed(456) # set the seed (to make the randomization repeatable)

# the for loop to fit 100 times the control data and to select the best fit:
for(i in 1:100){
	
	# Samples a start for r ranging from 0.0001 to 3:
	r.start = runif(1,min=0.0001,max=3) 
	
	# Samples a start for t0 from a value near to zero to a value slightly smaller than the day of the first infection:
	t0.start = runif(1,min=0.001,max=min(real.data.control$time.days[real.data.control$number.infected>0])*0.8)
	
	# sets the ith value to NA or infinity (i=1 in the first loop, 2 in the second and so on):
	r.control[i] = NA
	r.control.se[i] = NA
	r.control.z[i] = NA
	r.control.p[i] = NA
	t0.control[i] = NA
	t0.control.se[i] = NA
	t0.control.z[i] = NA
	t0.control.p[i] = NA
	AIC.control[i] = Inf
	# sets the object storing the fitting results to "NULL" to avoid a wrong entry from the loop before:
	fit.control = NULL
	
	# The try function allows a continuation of the for loop, even if the fit fails.
	try({
		# The fit:
		fit.control = mle2(mon.inf.lag.1p.nll,
			start = list(r = r.start, t0 = t0.start),
			data = list(nI = real.data.control$number.infected,
				tps = real.data.control$time.days,
				Imax = real.data.control$total.number
			),
			fixed = list(steps = 0.025), # decrease the temporal resolution of the lsoda function (increase the precisness of the curve)
			control = list(maxit=10000, reltol=1e-32) # increase the maximum number of iteration steps of the optimizer and decrease the tolerance
		)
		# save the summary statistics and the AIC to the vectors that you can display them later:
		r.control[i] = summary(fit.control)@coef[1]
		r.control.se[i] = summary(fit.control)@coef[3]
		r.control.z[i] = summary(fit.control)@coef[5]
		r.control.p[i] = summary(fit.control)@coef[7]
		t0.control[i] = summary(fit.control)@coef[2]
		t0.control.se[i] = summary(fit.control)@coef[4]
		t0.control.z[i] = summary(fit.control)@coef[6]
		t0.control.p[i] = summary(fit.control)@coef[8]
		AIC.control[i] = AIC(fit.control)
	})
	print(i) # which loop are we in?
	
}

# calculate the delta-AIC, note that comparing the likelihood would be also sufficient!
dAIC.control = AIC.control-min(AIC.control)

# displaying the fitting results of the best model:
r.control[dAIC.control==0]
r.control.se[dAIC.control==0]
r.control.z[dAIC.control==0]
r.control.p[dAIC.control==0]

t0.control[dAIC.control==0]
t0.control.se[dAIC.control==0]
t0.control.z[dAIC.control==0]
t0.control.p[dAIC.control==0]
	
##############################################################
## Fitting the treatent data with the 2-pathogen model:

# create empty vectors to store the fitting results in:
r.treat = vector() # for the value
r.treat.se = vector() # for the standard error
r.treat.z = vector() # for the z value
r.treat.p = vector() # for the p-value
t0.treat = vector() # for the value
t0.treat.se = vector() # for the standard error
t0.treat.z = vector() # for the z value
t0.treat.p = vector() # for the p-value
AIC.treat = vector() # for the AIC

set.seed(789) # set the seed (to make the randomization repeatable)

# the for loop to fit 100 times the control data and to select the best fit:
for(i in 1:100){
	
	# Samples a start for r ranging from 0.0001 to 15:
	r.start = runif(1,min=0.01,max=15)
	
	# Samples a start for t0 from a value near to zero to a value slightly smaller than the day of the first infection:
	t0.start = runif(1,min=0.001,max=min(real.data.treat$time.days[real.data.treat$number.infected>0])*0.8)
	
	# sets the ith value to NA or infinity (i=1 in the first loop, 2 in the second and so on):
	r.treat[i] = NA
	r.treat.se[i] = NA
	r.treat.z[i] = NA
	r.treat.p[i] = NA
	t0.treat[i] = NA
	t0.treat.se[i] = NA
	t0.treat.z[i] = NA
	t0.treat.p[i] = NA
	AIC.treat[i] = Inf
	# sets the object storing the fitting results to "NULL" to avoid a wrong entry from the loop before:
	fit.treat = NULL
	
	# The try function allows a continuation of the for loop, even if the fit fails.
	try({
		# The fit:
		fit.treat = mle2(mon.inf.lag.2p.nll,
			start = list(rp = r.start, t0p = t0.start),
			data = list(nI = real.data.treat$number.infected,
				tps = real.data.treat$time.days,
				Imax = real.data.treat$total.number
			),
			fixed = list(rc = r.control[dAIC.control==0]
						,t0c = t0.control[dAIC.control==0]
						,steps = 0.025), # decrease the temporal resolution of the lsoda function (increase the precisness of the curve)
			control = list(maxit=10000) # increase the maximum number of iteration steps of the optimizer and decrease the tolerance
		)
		# save the summary statistics and the AIC to the vectors that you can display them later:
		r.treat[i] = summary(fit.treat)@coef[1]
		r.treat.se[i] = summary(fit.treat)@coef[3]
		r.treat.z[i] = summary(fit.treat)@coef[5]
		r.treat.p[i] = summary(fit.treat)@coef[7]
		t0.treat[i] = summary(fit.treat)@coef[2]
		t0.treat.se[i] = summary(fit.treat)@coef[4]
		t0.treat.z[i] = summary(fit.treat)@coef[6]
		t0.treat.p[i] = summary(fit.treat)@coef[8]
		AIC.treat[i] = AIC(fit.treat)
	})
	print(i) # which loop are we in?
	
}

# calculate the delta-AIC, note that comparing the likelihood would be also sufficient!
dAIC.treat = AIC.treat-min(AIC.treat)

# displaying the fitting results of the best model:
r.treat[dAIC.treat==0]
r.treat.se[dAIC.treat==0]
r.treat.z[dAIC.treat==0]
r.treat.p[dAIC.treat==0]

t0.treat[dAIC.treat==0]
t0.treat.se[dAIC.treat==0]
t0.treat.z[dAIC.treat==0]
t0.treat.p[dAIC.treat==0]

##############################################################
### create a nice plot: ###

## first, calculate the regression lines, see manual for details:

xvalues = seq(0,10,length=100) ## x-values for the regression lines

## calculates the y-values according to the simple treatment fit
## for Imax = 6 (timeseries "3" in the data file):
treat.simple.est.6 =  data.frame(lsoda(y = c(I = 0),
		times = xvalues,
		func = mon.inf.lag.1p,
		parms = c(r = r.treat.simple[dAIC.treat.simple==0],
			t0 = t0.treat.simple[dAIC.treat.simple==0],
			Imax = 6)
	)
)

## calculates the y-values according to the simple treatment fit
## for Imax = 8 (timeseries "2" and "4" in the data file):
treat.simple.est.8 =  data.frame(lsoda(y = c(I = 0),
		times = xvalues,
		func = mon.inf.lag.1p,
		parms = c(r = r.treat.simple[dAIC.treat.simple==0],
			t0 = t0.treat.simple[dAIC.treat.simple==0],
			Imax = 8)
	)
)

## calculates the y-values according to the control fit:
control.est =  data.frame(lsoda(y = c(I = 0),
		times = xvalues,
		func = mon.inf.lag.1p,
		parms = c(r = r.control[dAIC.control==0],
			t0 = t0.control[dAIC.control==0],
			Imax = 8)
	)
)

## calculates the y-values according to the 2-pathogen treatment fit
## for Imax = 6 (timeseries "3" in the data file):
treat.est.6 =  data.frame(lsoda(y = c(Ip = 0, Ic = 0),
		times = xvalues,
		func = mon.inf.lag.2p,
		parms = c(rp = r.treat[dAIC.treat==0],
			t0p = t0.treat[dAIC.treat==0],
			rc = r.control[dAIC.control==0],
			t0c = t0.control[dAIC.control==0],
			Imax = 6
		)
	)
)

## calculates the y-values according to the 2-pathogen treatment fit
## for Imax = 8 (timeseries "2" and "4" in the data file):
treat.est.8 =  data.frame(lsoda(y = c(Ip = 0, Ic = 0),
		times = xvalues,
		func = mon.inf.lag.2p,
		parms = c(rp = r.treat[dAIC.treat==0],
			t0p = t0.treat[dAIC.treat==0],
			rc = r.control[dAIC.control==0],
			t0c = t0.control[dAIC.control==0],
			Imax = 8
		)
	)
)
	
## The plotting commands. Please see the regular text books and help files for details (e.g. Crawley (2007): The R-Book)

par(mfrow = c(1,3), cex.axis=0.75,mar=c(4,4,.5,.5),las=1) ## graphical settings, 1x2 plotting regions with reduced size of axis labels
plot(real.data.control$time.days,
	real.data.control$number.infected,
	xlim=c(0,10),
	ylim=c(0,10),
	xlab="days",
	ylab="infections",
	pch=16)
mtext(side=3,"(A)",line=-1.5,adj=0.03)
lines(control.est$time, control.est$I, col = "#FFB3B3", lwd = 2)

plot(real.data.treat$time.days[real.data.treat$total.number==6],
	real.data.treat$number.infected[real.data.treat$total.number==6],
	xlim=c(0,10),
	ylim=c(0,10),
	xlab="days",
	ylab="infections",
	pch=16)
mtext(side=3,"(B)",line=-1.5,adj=0.03)
lines(treat.est.6$time,
	treat.est.6$Ip+treat.est.6$Ic,
	lwd = 2)
lines(treat.simple.est.6$time,
		treat.simple.est.6$I,
		col = "orange",
		lty = 2,
		lwd = 2)

plot(real.data.treat$time.days[real.data.treat$total.number==8],
	real.data.treat$number.infected[real.data.treat$total.number==8],
	xlim=c(0,10),
	ylim=c(0,10),
	xlab="days",
	ylab="infections",
	pch=16)
mtext(side=3,"(C)",line=-1.5,adj=0.03)
lines(treat.est.8$time,
	treat.est.8$Ip+treat.est.8$Ic,
	lwd = 2)
lines(treat.simple.est.8$time,
		treat.simple.est.8$I,
		col = "orange",
		lty = 2,
		lwd = 2)
