	
rm(list=ls()) ## removes old list objects to clean workspace
	
setwd("~/MainSources/") ## set your working directory
	
library("deSolve") ## loads required package to solve ODEs, must be eventually installed
library("bbmle") ## loads required package to perform maximum likelihood fit, must be eventually installed
	
source("source/infection.models.r") ## loads source file with ODEs
source("source/infection.nll.r") ## loads source file with negative likelihood functions
	
no.of.data.reps = 4 ## number of replicates per time point
no.of.reps = 1100 ## number of differently simulated time series
time.of.exp = 30 ## length of experiment, here 30 time steps (can be days or hours, ...)
	
set.seed(665) ## sets the seed to fix the start value of the random number generators -> creates reproducable data
	
r.control = runif(no.of.reps,0.001,0.1) ## samples infection rates of the natural occuring pathogen
t0.control = runif(no.of.reps,1,5) ## samples resistance times of the natural occuring pathogen
	
r.treat.pathogen = runif(no.of.reps,0.1,0.5) ## samples infection rates of the treatment pathogen
t0.treat.pathogen = runif(no.of.reps,5,10) ## samples resistance times of the treatment pathogen
	
r.control.est = vector() ## empty vector in which the fitting results will be saved in
t0.control.est = vector() ## empty vector in which the fitting results will be saved in
	
r.one.pathogen.est = vector() ## empty vector in which the fitting results will be saved in
t0.one.pathogen.est = vector() ## empty vector in which the fitting results will be saved in
	
r.two.pathogen.est = vector() ## empty vector in which the fitting results will be saved in
t0.two.pathogen.est = vector() ## empty vector in which the fitting results will be saved in
	
exp.times = sort(rep(seq(1,time.of.exp,1),no.of.data.reps)) ## time steps in which a sample should be taken, here each time step
	
## for-loop to simulate and estimate the 1100 replicates of the scenario
for(j in 1:no.of.reps){
	
	sim1 = NULL ## clears object sim1, the control simulation will be save in there
	sim2 = NULL ## clears object sim2, the treatment simulation will be save in there
	exp.nI = vector() ## creates empty vector to save the infections from the control simulation
	exp.nI2 = vector() ## creates empty vector to save the infections from the treatment simulation
	fit.one.pathogen = NULL ## clears object fit.one.pathogen, the fit of the one-pathogen model to the treatment data will be saved in here
	fit.control = NULL ## clears object fit.control, the fit of the one-pathogen model to the control data will be saved in here
	fit.two.pathogens = NULL ## clears object fit.two.pathogen, the fit of the two-pathogen model to the treatment data will be saved in here
	
	r.control.est[j] = -999.9 ## sets the infection rate measure for the control fit to -999.9 at position "j", allows for exluding failed fits
	t0.control.est[j] = -999.9 ## sets the resistance time measure for the control fit to -999.9 at position "j", allows for exluding failed fits
	
	r.one.pathogen.est[j] = -999.9 ## sets the infection rate measure for the one-pathogen fit to -999.9 at position "j", allows for exluding failed fits
	t0.one.pathogen.est[j] = -999.9 ## sets the resistance time measure for the one-pathogen fit to -999.9 at position "j", allows for exluding failed fits
	
	r.two.pathogen.est[j] = -999.9 ## sets the infection rate measure for the two-pathogen fit to -999.9 at position "j", allows for exluding failed fits
	t0.two.pathogen.est[j] = -999.9 ## sets the resistance time measure for the two-pathogen fit to -999.9 at position "j", allows for exluding failed fits
	
	#######################################################################################################################################################################
	############## simulation of data using lsoda, see the attached How-To, ###############################################################################################
	############## Soetart et al. (2010): Solving differential equations in R: package deSolve; JSS 33(9); (http://www.jstatsoft.org/article/view/v033i09) ################
	############## and Soetart and Herman (2008): A Practical Guide to Ecological Modelling: Using R as a Simulation Platform #############################################
	############## for details ############################################################################################################################################
	#######################################################################################################################################################################
	## simulation of control time series:
	sim1 = data.frame(lsoda(y = c(I = 0), ## starting density of infected plants is set to "0"
				times = seq(0,time.of.exp,0.01), ## time steps of the time series, here set to 0.01
				func = mon.inf.lag.1p, ## calls the required function from the source file
				parms = c(r = r.control[j], Imax = 10, t0 = t0.control[j]) ## parameter values, Imax is set to 10 plants
				))
	## for each sample out of the times series 4 data points are generated, infections are distributed according to a binomial distribution
	for(i in 1:length(exp.times)){
		exp.nI[i] = rbinom(1,10,sim1$I[sim1$time==exp.times[i]]/10)
	}
	## simulation of treatment time series:
	sim2 = data.frame(lsoda(y = c(Ip = 0, Ic = 0), ## starting density of infected plants by treatment pathogen (Ip) or control pathogen (Ic) is set to "0"
				times = seq(0,time.of.exp,0.01), ## time steps of the time series, here set to 0.01
				func = mon.inf.lag.2p, ## calls the required function from the source file
				parms = c(rp = r.treat.pathogen[j], rc = r.control[j], Imax = 10, t0p = t0.treat.pathogen[j], t0c = t0.control[j]) ## parameter values, Imax is set to 10 plants
				))
	## for each sample out of the times series 4 data points are generated, infections are distributed according to a binomial distribution
	for(i in 1:length(exp.times)){
		exp.nI2[i] = rbinom(1,10,(sim2$Ip[sim2$time==exp.times[i]]+sim2$Ic[sim2$time==exp.times[i]])/10)
	}
	
	#######################################################################################################################################################################
	############## fitting the models to the data using mle2, see the attached How-To, ####################################################################################
	############## Bolker & the R Development Core Team (2014): bbmle: Tools for general maximum likelihood estimation ####################################################
	############## and Bolker (2008): Ecological Models and Data in R #####################################################################################################
	############## for details ############################################################################################################################################
	#######################################################################################################################################################################
	## fitting of the one-pathogen model to the treatment data
	## the try-function enables the fit to fail while the for-loop continues
	try({
		fit.one.pathogen = mle2(mon.inf.lag.1p.nll, ## the required negative likelihood function 
			start = list(r = r.treat.pathogen[j]/2, t0 = t0.control[j]*0.75), ## nice starting values for the iterative fit
			data = list(nI = exp.nI2,
						tps = exp.times
			), ## hand over the data
			fixed = list(Imax = 10), ## we fix the maximum number of plants that can be infected
			control = list(reltol=1e-16, maxit = 2000) ## we reduce the tolerance to get a more precise fit, and we increase the maximum of allowed iterations 
		)
		r.one.pathogen.est[j] = summary(fit.one.pathogen)@coef[1] ## saves fitted value
		t0.one.pathogen.est[j] = summary(fit.one.pathogen)@coef[2] ## saves fitted value
	})
	#summary(fit.one.pathogen) ## activate if you wanna see the output, avoid if you run the for-loop
	
	## fitting of the one-pathogen model to the control data
	try({
		fit.control = mle2(mon.inf.lag.1p.nll, ## the required negative likelihood function
			start = list(r = r.control[j]/2, t0 = t0.control[j]*0.75), ## nice starting values for the iterative fit
			data = list(nI = exp.nI,
						tps = exp.times
			), ## hand over the data
			fixed = list(Imax = 10), ## we fix the maximum number of plants that can be infected
			control = list(reltol=1e-16, maxit = 2000) ## we reduce the tolerance to get a more precise fit, and we increase the maximum of allowed iterations 
		)
		r.control.est[j] = summary(fit.control)@coef[1] ## saves fitted value
		t0.control.est[j] = summary(fit.control)@coef[2] ## saves fitted value
	})
	#summary(fit.control) ## activate if you wanna see the output, avoid if you run the for-loop
	
	## fitting of the two-pathogen model to the treatment data
	try({
	fit.two.pathogens = mle2(mon.inf.lag.2p.nll, ## the required negative likelihood function
		start = list(rp = r.treat.pathogen[j]/2, t0p = t0.control[j]*0.75), ## nice starting values for the iterative fit
		data = list(nI = exp.nI2,
					tps = exp.times
		), ## hand over the data
		fixed = list(Imax = 10,
			rc = coef(fit.control)[[1]],
			t0c = coef(fit.control)[[2]]
		), ## we fix the maximum number of plants that can be infected, also the control estimates for infection rates and resistance time are fixed
		control = list(reltol=1e-16, maxit = 2000) ## we reduce the tolerance to get a more precise fit, and we increase the maximum of allowed iterations
	)
		r.two.pathogen.est[j] = summary(fit.two.pathogens)@coef[1] ## saves fitted value
		t0.two.pathogen.est[j] = summary(fit.two.pathogens)@coef[2] ## saves fitted value
	})
	#summary(fit.two.pathogens) ## activate if you wanna see the output, avoid if you run the for-loop
	
	print(j) ## print out the iteration number of the for-loop
}
	
## creates a data frame with all the results from above:
fit.results = data.frame(r.control = r.control,
							t0.control = t0.control,
							r.treat.pathogen = r.treat.pathogen,
							t0.treat.pathogen = t0.treat.pathogen,
							r.control.est = r.control.est,
							t0.control.est = t0.control.est,
							r.one.pathogen.est = r.one.pathogen.est,
							t0.one.pathogen.est = t0.one.pathogen.est,
							r.two.pathogen.est = r.two.pathogen.est,
							t0.two.pathogen.est = t0.two.pathogen.est
							)
	
write.csv(fit.results,"data/data_scenario01.csv", row.names = F) ## writes the data table for further investigation
	
